#!/usr/bin/perl

use Getopt::Std;
use MP3::Info;

if($#ARGV < 0) {
    print "Usage: rename.pl [options] files...\n";
    print "Options:\n";
    print "\t-a [artist] - Specify artist\n";
    print "\t-l [album]  - Specify album name\n";
    print "\t-t          - Use hacked ID3 track number (last byte of track name)\n";
    print "\t-A [regex]  - Specify how to find artist name in filename\n";
    print "\t-L [regex]  - Specify how to find album name in filename\n";
    print "\t-N [regex]  - Specify how to find track name in filename\n";
    print "\t-T [regex]  - Specify how to find track number in filename\n";
    print "\t-z          - Swap artist namn and album name (useful for compilations)\n";
    print "\t-w          - Track number on position 2 (useful for compilations)\n";
    print "\t-i		 - Don't use ID3 tags (guess from filename)\n";
    print "\t-r          - Perform the rename\n";
    exit 0;
}

$genre = 12;

getopts('a:l:A:L:N:T:trzwi', \%opt) or exit;

foreach $file (@ARGV) {
    ($dummy, $track_name, $artist_name, $album_name, $year,
     $comment, $track_num, $genre) = "";

    if( !$opt{'i'} ) {
        my $tags = get_mp3tag($file);

        $track_name = $tags->{TITLE} if ( $tags->{TITLE} );
        $artist_name = $tags->{ARTIST} if ( $tags->{ARTIST} );
        $album_name = $tags->{ALBUM} if ( $tags->{ALBUM} );
        $track_num = $tags->{TRACKNUM} if ( $tags->{TRACKNUM} );
    }

    # handle options
    $artist_name = $opt{'a'} if ( $opt{'a'} );
    $album_name = $opt{'l'} if ( $opt{'l'} );
    if ( $opt{'A'} and $file =~ /$opt{'A'}/ ) {
        $artist_name = $1;
    }
    if ( $opt{'L'} and $file =~ /$opt{'L'}/ ) {
        $album_name = $1;
    }
    if ( $opt{'N'} and $file =~ /$opt{'N'}/ ) {
        $track_name = $1;
    }
    if ( $opt{'T'} and $file =~ /$opt{'T'}/ ) {
        $track_num = $1;
    }

    # try to find track number, if not found yet
    if ( !$track_num ) {
        if ( $file =~ / ?- ?(\d\d) ?- ?/ ) { # " - 12 - "
            $track_num = $1;
        }
        elsif ( $file =~ /(\d+) ?of ?\d+/i ) { # "12 of 18"
            $track_num = $1;
        }
        elsif ( $file =~ /\D(\d+) ?[\/_] ?\d+/ ) { # "12 / 18"
            $track_num = $1;
        }
        elsif ( $file =~ /^(\d\d)\D/ ) {
            $track_num = $1;
        }
        elsif ( $file =~ /\D(\d\d)\D/ ) {
            $track_num = $1;
        }
    }

    # give a shot at "artist - album - track# - track name"
    if ( $file =~ /([^-]+) - ([^-]+) - (\d+) - ([^-]+).mp3/i ) {
        $artist_name = $1 if ( !$artist_name );
        $album_name  = $2 if ( !$album_name );
        $track_num   = $3 if ( !$track_num );
        $track_name  = $4 if ( !$track_name );

        $jox = $4;
    }
    elsif ( $file =~ /([^-]+) - ([^-]+) - ([^-]+).mp3/i ) {
        $artist_name = $1 if ( !$artist_name );
        $album_name  = $2 if ( !$album_name );
        $track_name  = $3 if ( !$track_name );

        $jox = $3;
    }

    # filenames are longer than id3's
    if ( length($track_name) == 30 and length($jox)>30 ) {
        $track_name = $jox;
    }
    $jox ="";

    # try to find song name (mostly last)
    if ( $track_name eq "" ) {
        $file =~ /\s?([^-]*).mp3/i;
        $track_name = $1;
    }


    # strip trailing spaces and null bytes
    $track_name =~ s/(.*?)[\0\s]+$/$1/;
    $artist_name =~ s/(.*?)[\0\s]+$/$1/;
    $album_name =~ s/(.*?)[\0\s]+$/$1/;
    $comment =~ s/(.*?)[\s\0]+$/$1/;

    # Strip path information
    $album_name =~ s/^.*[\\\/]+//g;
    
    # Remove track numbers, unless explicitly specified
    if ( !$opt{'l'} and !$opt{'L'} ) {
        $album_name =~ s/^([0-9]+) *//;
    }
    if ( !$opt{'N'} ) {
        $track_name =~ s/^([0-9]+) - //;
    }

    if ( $opt{'z'} ) {
        $temp = $album_name;
        $album_name = $artist_name;
        $artist_name = $temp;
    }

    $track_num = sprintf("%02d", $track_num);

    if ( $track_num ) {
        if ( $opt{'w'} ) {
            $newfile = "$artist_name - $track_num - $album_name - $track_name.mp3";
        }
        else {
            $newfile = "$artist_name - $album_name - $track_num - $track_name.mp3";
        }
    }
    else {
        $newfile = "$artist_name - $album_name - $track_name.mp3";
    }

    if ( $newfile =~ /\// ) {
        $newfile =~ s/(\/(.+?)).mp3/ \($2\).mp3/;
    }

    if ( $file eq $newfile ) {
        print "No change: $file\n";
    }
    else {
        if ( -f $newfile ) {
            print STDOUT "WARNING: '$newfile' already exists!\n";
        }
        else {
            #print "$file =>\n$newfile\n";
            print "$newfile\n";
        }
        if ( $opt{'r'} ) {
            rename( $file, $newfile ) or print STDOUT "$newfile: $!\n";
        }
    }
}
